 '-----------------------------------------------------------------------
 '                        R A N D G E N
 '-----------------------------------------------------------------------
 ' Performance test of 2 algorithms about random numbers generation.
 '-----------------------------------------------------------------------
 ' Input files: RandGen.scn
 ' Process: Conversion from DOS format to WIN32 console format
 '          of RandGen.scn and display on console.
 '          Generation of 5000 rows of 64 numbers using 2 algorithms:
 '          - 1 - Numbers generation in PB
 '          - 2 - Numbers generation in ASM
 ' Output:  Display the number of cycles used by each algorithm
 '          the generation progress and the MS elapsed.
 '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 ' Date: 24-apr-2010
 ' Language: PB/CC 5.04
 ' Author: frktons @ MASM32 Forum
 '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


 #INCLUDE "win32api.inc"


 DECLARE SUB ReadForm(BYVAL location AS LONG)

 DECLARE SUB DisplayScreen

 DECLARE SUB TestPB
 DECLARE SUB GenRowPB
 DECLARE SUB TestASM

 DECLARE FUNCTION GetTickCount LIB "KERNEL32.DLL" ALIAS "GetTickCount" () AS DWORD

 GLOBAL n&(), g&()

 FUNCTION PBMAIN()

   GLOBAL MainStr AS STRING, sBuf AS STRING
   GLOBAL QtGen AS LONG, location AS LONG, Rows AS LONG
   GLOBAL x,x1,x2,x3,x4,x5,x6,y,y1,y2,y3,y4,y5,y6 AS LONG
   DIM n&(1 TO 5000, 1 TO 64)
   DIM g&(1 TO 4)
   GLOBAL Min_n AS LONG
   GLOBAL Max_n AS LONG
   GLOBAL ok, si, no AS LONG
   LOCAL tc AS DWORD

   ok = 0
   si = -1
   no = 0

   Min_n = 1
   Max_n = 50

   CONSOLE SCREEN 25,80

   CURSOR OFF

   CALL ReadForm(1)
   CALL ConvertASM
   CALL DisplayScreen

   LOCATE 23,30
   COLOR 0,8
   PRINT "Generating random numbers - PB version..."

   tc = GetTickCount
   TIX cycles&&
       CALL TestPB
   TIX END cycles&&
   tc = GetTickCount - tc

   LOCATE 13,24
   PRINT FORMAT$(cycles&&,"###,");

   LOCATE 16,24
   PRINT FORMAT$(tc,"###,");

   LOCATE 23,30
   COLOR 0,8
   PRINT "Random numbers generation complete - PB version"

   WAITKEY$

'   goto exit_pgm

'   TIX cycles&&
'       CALL TestASM
'   TIX END cycles&&
'
'   LOCATE 13,32
'   PRINT FORMAT$(cycles&&,"###,");
'
'   WAITKEY$

 exit_pgm:

 END FUNCTION

'--------------------------------------------------------------------------------
' Generate 5000 rows of 64 integer numbers - PB version
'--------------------------------------------------------------------------------

 SUB TestPB

    FOR x = 1 TO 5000
        CALL GenRowPB
        LOCATE 10,21
        PRINT FORMAT$(x,"###,")
    NEXT x

 END SUB

'--------------------------------------------------------------------------------
' Generate single row of 64 integer numbers - PB version
'--------------------------------------------------------------------------------

 SUB GenRowPB

    RANDOMIZE TIMER

    FOR y = 1 TO 16

    again:

        g&(1) = RND(Min_n,Max_n)
        g&(2) = RND(Min_n,Max_n)
        g&(3) = RND(Min_n,Max_n)
        g&(4) = RND(Min_n,Max_n)

        ok = si

        FOR x2 = 1 TO 3
            FOR x3 = x2 + 1 TO 4
                IF g&(x2) = g&(x3) THEN
                    ok = no
                    EXIT
                END IF
            NEXT x3
            IF ok = no THEN
                EXIT
            END IF
         NEXT x2
         IF ok = no THEN
             GOTO again
         END IF
    NEXT y

 END SUB
'--------------------------------------------------------------------------------
' Display the converted Screen
'--------------------------------------------------------------------------------

 SUB DisplayScreen

    LOCAL lpReadRegion AS SMALL_RECT
    LOCAL SIZE AS DWORD
    SIZE = MAKDWD(80, 25)

    lpReadRegion.xRight = 79
    lpReadRegion.xBottom = 24
    WriteConsoleOutPut GetStdHandle(%STD_OUTPUT_HANDLE), _
    BYVAL STRPTR(sBuf),BYVAL SIZE, BYVAL 0&, lpReadRegion
 END SUB

'--------------------------------------------------------------------------------
' Read the DOS screen into the variabile MainStr
'--------------------------------------------------------------------------------

 SUB ReadForm(BYVAL location AS LONG)
    LOCAL ff AS LONG

    ff = FREEFILE
    OPEN "RandGen.scn" FOR BINARY AS #ff
      SEEK ff, location
      GET$ #ff, 4000, MainStr
    CLOSE #ff

 END SUB


'--------------------------------------------------------------------------------
' Convert the DOS screen into WIN Console screen - ASM version
'--------------------------------------------------------------------------------
 SUB ConvertASM

    #REGISTER NONE

    ! push ebx
    ! push esi
    ! push edi
    ! push eax

    sBuf = SPACE$(8000)

       ! mov ebx, 1000
       ! mov esi, MainStr
       ! mov edi, sBuf
       Convert:
       ! mov eax, [esi]
       ! mov [edi], al
       ! mov [edi+2], ah
       ! bswap eax
       ! mov [edi+6], al
       ! mov [edi+4], ah
       ! add esi, 4
       ! add edi, 8
       ! dec ebx
       ! jnz Convert

    ! pop eax
    ! pop edi
    ! pop esi
    ! pop ebx

  END SUB
